function R = shrinkCenters_VER2(C,CENTERS,rho)
% SHRINKCENTERS_VER2 computes the 3D estimation of the particles's center.
%
%   R = shrinkCenters_VER2(C,CENTERS,rho)
%
% It searches in C the nearest neighbours of CENTERS within a distance rho and
% then compute the estimation via a weighted mean.
%
% INPUT
%
% C       :(n-by-4 double) the first 3 columns contain the 3d coordinates, 
%           the 4th the weight used for computing the mean
%
% CENTERS :(m-by-3 double) raw estimation of the center obtained 
%
% rho     :(double) distance within which the nearest neighbours in C to
%          CENTERS are searched
%
% OUTPUT
%
% R       :(m-by 3 double) 3D coordinates of the final estimation of the centers
%
% See also mergingSPHERE
%
%==========================================================================
%
% Version : 1.2 (06-06-2019)
% Author  : A. Benfenati (alessandro.benfenati@unimi.it)
%
%==========================================================================
%
% COPYRIGHT NOTIFICATION
%
% Permission to copy and modify this software and its documentation for
% internal research use is granted, provided that this notice is retained
% thereon and on all copies or modifications. The authors and their
% respective Universities makes no representations as to the suitability
% and operability of this software for any purpose. It is provided "as is"
% without express or implied warranty. Use of this software for commercial
% purposes is expressly prohibited without contacting the authors.
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 3 of the License, or (at your
% option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.
%
% You should have received a copy of the GNU General Public License along
% with this program; if not, either visite http://www.gnu.org/licenses/ or
% write to Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
% 02139, USA.
%


% Allocating name
R       = [];

% Search for the rho--nearest neaighbours in C
A = rangesearch(C(:,1:3),CENTERS,rho);

% For each group of neighbours, compute the weighted mean
for i = 1:length(A)
    if length(A{i})>1
        A{i} = unique(A{i});
        W = C(A{i},4);
        R = [R; sum(diag(W)*C(A{i},1:3))/sum(W)];
    end
end
